
from fastapi import Depends, status, Response, HTTPException
from fastapi.security import OAuth2PasswordBearer
from passlib.context import CryptContext
from pydantic import BaseModel
import models
from .auth_token import verify_token
from sqlalchemy.orm import Session
from database import get_db
from models.UserModel import User 


oauth2_scheme = OAuth2PasswordBearer(tokenUrl="login")

def get_current_user(token: str = Depends(oauth2_scheme), db: Session = Depends(get_db)):
    credentials_exception = HTTPException(
        status_code=status.HTTP_401_UNAUTHORIZED,
        detail="Could not validate credentials", 
        headers={"WWW-Authenticate": "Bearer"},
    )

    token_data = verify_token(token)
    user = db.query( User).filter( User.username == token_data.username).first()

    if not user:
        raise credentials_exception

    return user